import re
from typing import Dict

from aws_lambda_powertools.utilities.validation import SchemaValidationError
from chocs import HttpRequest, Route
from openapi_spec_validator import validate_spec as oas_validator
from opyapi.errors import ValidationError

from powertools_oas_validator.services.request_validator import RequestValidatorABC
from powertools_oas_validator.services.spec_loader import SpecLoaderProtocol


class SpecValidator:
    def __init__(
        self,
        file_path: str,
        event: Dict,
        spec_loader: SpecLoaderProtocol,
        request_validator: RequestValidatorABC,
    ) -> None:
        self.file_path = file_path
        self.event = event
        self.spec_loader = spec_loader
        self.request_validator = request_validator

    def validate_spec(self) -> None:
        loaded_spec = self.spec_loader.read_from_file_name(self.file_path)

        # Raises OpenAPIValidationError on error
        oas_validator(spec=loaded_spec.spec_dict, spec_url=loaded_spec.spec_url)

    def validate_request_against_spec(self) -> None:
        request = self._event_to_request(self.event)

        validator_cache_key = f"{request.method} {request.path}".lower()
        if validator_cache_key not in self.request_validator._validators:
            self.request_validator._validators[
                validator_cache_key
            ] = self.request_validator.get_validators_for_uri(
                request.path,
                request.method,
                str(request.headers.get("content-type", "application/json")),
            )

        validators = self.request_validator._validators[validator_cache_key]
        if not validators:
            return
        for validator in validators:
            # Raises Validation Error
            # https://github.com/kodemore/chocs-openapi/blob/main/chocs_middleware/openapi/error.py
            try:
                validator(request)
            except ValidationError as ex:
                raise self.validator_error_to_schema_validaton_error(ex, request)

    def _event_to_request(self, event: Dict) -> HttpRequest:
        req = HttpRequest(
            path=self.request_validator.get_path(event),
            method=self.request_validator.get_method(event),
            headers=self.request_validator.get_headers(event),
            body=self.request_validator.get_body(event),
            query_string=self.request_validator.get_query_string(event),
        )

        req._cookies = self.request_validator.get_cookies(event)
        req.route = Route(req.path)

        return req

    def validator_error_to_schema_validaton_error(
        self, ex: ValidationError, request: HttpRequest
    ) -> SchemaValidationError:
        violating_param = re.search("`(.+?)`", ex.context["reason"])
        violating_param = violating_param.group(1)  # type: ignore

        name = f"paths.{request.path.lstrip('/')}.{request.method.value.lower()}"

        code_to_oas_path = {
            "invalid_request_body": "requestBody",
            "invalid_request_query": "parameters",
            "invalid_request_headers": "parameters",
            "invalid_request_uri": "parameters",
        }

        try:
            name = name + f".{code_to_oas_path[ex.code]}"
        except KeyError:
            ...  # No support for invalid_request_cookies

        name = name + f".[{violating_param}]"
        path = name.replace("[", "").replace("]", "").split(".")

        return SchemaValidationError(
            name=name,
            path=path,
            validation_message=ex.context["reason"],
            definition=ex.message,
        )



from collections.abc import Callable
from typing import Dict

from aws_lambda_powertools.middleware_factory import lambda_handler_decorator
from aws_lambda_powertools.utilities.typing import LambdaContext

from powertools_oas_validator.services.request_validator import RequestValidator
from powertools_oas_validator.services.spec_loader import SpecLoader
from powertools_oas_validator.services.spec_validator import SpecValidator
from powertools_oas_validator.types import ValidationConfig


@lambda_handler_decorator
def validate_request(
    handler: Callable,
    event: Dict,
    context: LambdaContext,
    oas_path: str,
    validate_spec=True,
    validate_body=True,
    validate_headers=True,
    validate_query=True,
    validate_path=True,
    validate_cookies=True,
) -> Callable:
    config = ValidationConfig(
        validate_body=validate_body,
        validate_headers=validate_headers,
        validate_query=validate_query,
        validate_path=validate_path,
        validate_cookies=validate_cookies,
    )

    # Validate File Path
    spec_loader = SpecLoader()
    spec_loader.validate_file(oas_path)

    # Setup Request Validator
    request_validator = RequestValidator(oas_path, config)

    # Setup SpecValidator
    spec_validator = SpecValidator(oas_path, event, spec_loader, request_validator)

    # If Spec Should be validated against OpenAPI Schema standard
    if validate_spec:
        spec_validator.validate_spec()

    # Validate Spec Against Request
    spec_validator.validate_request_against_spec()

    return handler(event, context)



from dataclasses import dataclass
from typing import Any, Dict, Hashable, Mapping


@dataclass
class LoadedSpec:
    spec_dict: Mapping[Hashable, Any]
    spec_url: str


class ValidationConfig:
    config: Dict[str, bool]

    def __init__(
        self,
        validate_body: bool = True,
        validate_headers: bool = True,
        validate_query: bool = True,
        validate_path: bool = True,
        validate_cookies: bool = True,
    ) -> None:
        self.config = {
            "body": validate_body,
            "headers": validate_headers,
            "query": validate_query,
            "path": validate_path,
            "cookies": validate_cookies,
        }



import json
from abc import abstractmethod
from typing import Callable, Dict, List, Optional

from chocs import HttpMethod  # type: ignore
from chocs import (
    HttpCookie,
    HttpCookieJar,
    HttpHeaders,
    HttpQueryString,
    HttpRequest,
    HttpResponse,
)
from chocs.middleware import MiddlewareHandler  # type: ignore
from chocs_middleware.openapi.middleware import OpenApiMiddleware
from opyapi import JsonSchema

from powertools_oas_validator.exceptions import InvalidEventError
from powertools_oas_validator.types import ValidationConfig


class MiddlewareWrapper(OpenApiMiddleware):
    def handle(self, request: HttpRequest, next: MiddlewareHandler) -> HttpResponse:
        raise NotImplementedError  # pragma: nocover

    def get_validators_for_uri(
        self, route: str, method: HttpMethod, content_type: str
    ) -> List[Callable]:
        return self._get_validators_for_uri(route, method, content_type)


class RequestValidatorABC(MiddlewareWrapper):
    @abstractmethod
    def get_path(self, event: Dict) -> str:
        raise NotImplementedError  # pragma: nocover

    @abstractmethod
    def get_method(self, event: Dict) -> HttpMethod:
        raise NotImplementedError  # pragma: nocover

    @abstractmethod
    def get_headers(self, event: Dict) -> Optional[HttpHeaders]:
        raise NotImplementedError  # pragma: nocover

    @abstractmethod
    def get_query_string(self, event: Dict) -> Optional[HttpQueryString]:
        raise NotImplementedError  # pragma: nocover

    @abstractmethod
    def get_body(self, event: Dict) -> Optional[str]:
        raise NotImplementedError  # pragma: nocover

    @abstractmethod
    def get_cookies(self, event: Dict) -> HttpCookieJar:
        raise NotImplementedError  # pragma: nocover


class RequestValidator(RequestValidatorABC):
    def __init__(self, file_path: str, validation_config: ValidationConfig) -> None:
        self.openapi = JsonSchema.from_file(file_path)
        self.validators = validation_config.config
        self._validators: Dict[str, List[Callable]] = {}

    def get_path(self, event: Dict) -> str:
        try:
            return event["resource"]
        except KeyError:
            raise InvalidEventError("'resource' not in event")

    def get_method(self, event: Dict) -> HttpMethod:
        try:
            return HttpMethod[event["httpMethod"]]
        except KeyError:
            raise InvalidEventError("'httpMethod' not in event")

    def get_headers(self, event: Dict) -> Optional[HttpHeaders]:
        if "headers" in event:
            return HttpHeaders(event["headers"])
        else:
            return None

    def get_query_string(self, event: Dict) -> Optional[HttpQueryString]:
        if "rawQueryString" in event:
            return HttpQueryString(event["rawQueryString"])

        else:
            return None

    def get_body(self, event: Dict) -> Optional[str]:
        if "body" in event:
            if type(event["body"]) == dict:
                return json.dumps(event["body"])
            else:
                return event["body"]
        else:
            return None

    def get_cookies(self, event: Dict) -> HttpCookieJar:
        cookie_jar = HttpCookieJar()
        if "cookies" in event:
            for event_cookie in event["cookies"]:
                cookies = event_cookie.replace(" ", "").split(";")

                for cookie in cookies:
                    values = cookie.split("=")

                    if len(values) == 1:
                        value = ""
                    else:
                        value = values[1]

                    cookie_jar.append(HttpCookie(values[0], value))
        return cookie_jar



import os
from typing import Protocol

from openapi_spec_validator.readers import read_from_filename

from powertools_oas_validator.exceptions import (
    FileNotExistsError,
    NotSupportedFileTypeError,
)
from powertools_oas_validator.types import LoadedSpec


class SpecLoaderProtocol(Protocol):
    def read_from_file_name(self, file_path: str) -> LoadedSpec:
        raise NotImplementedError  # pragma: nocover


class SpecLoader:
    validated_cache: bool = False

    def read_from_file_name(self, file_path: str) -> LoadedSpec:
        self.validate_file(file_path)

        spec_dict, spec_url = read_from_filename(file_path)

        return LoadedSpec(spec_dict=spec_dict, spec_url=spec_url)

    def validate_file(self, file_path: str) -> None:
        if self.validated_cache:
            return

        if not os.path.isfile(file_path):
            raise FileNotExistsError(f"File does not exist on path: {file_path}")

        extension = file_path.split(".")[-1]

        if extension not in ["yaml", "json"]:
            raise NotSupportedFileTypeError(
                f"'.{extension} not supported. Only '.json' and '.yaml'"
            )

        self.validated_cache = True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['powertools_oas_validator', 'powertools_oas_validator.services']

package_data = \
{'': ['*']}

install_requires = \
['aws-lambda-powertools>=2.18.0,<3.0.0',
 'chocs-middleware-openapi>=1.2.2,<2.0.0',
 'fastjsonschema>=2.17.1,<3.0.0',
 'jmespath>=1.0.1,<2.0.0',
 'openapi-spec-validator>=0.5.7,<0.6.0']

setup_kwargs = {
    'name': 'powertools-oas-validator',
    'version': '0.3.0',
    'description': '',
    'long_description': '# powertools-oas-validator\n<br>[![PyPI version](https://badge.fury.io/py/powertools-oas-validator.svg)](https://pypi.org/project/powertools-oas-validator/) ![Release](https://github.com/RasmusFangel/powertools-oas-validator/workflows/Publish/badge.svg) ![CI](https://github.com/RasmusFangel/powertools-oas-validator/workflows/CI/badge.svg)\n\n## Introduction\n\n[Powertools for AWS Lambda (Python)](https://github.com/aws-powertools/powertools-lambda-python) is an awesome set of tools for supercharging your lambdas. Powertools supports validating incoming requests (or event in PT lingo) against [JSONSchema](https://json-schema.org/) which is not ideal if you are using OpenAPI schemas to define your API contracts.\n\nThe *Powertools OAS Validator* adds a decorator that you can use with your lambda handlers and have the events validated against an OpenAPI schema instead.\n\n\n## Usage\nDecorate your functions with `@validate_request(oas_path="openapi.yaml")` and your request (and schema) will be validated on a request.\n\nSchema validation is enabled per default but can be disabled if you already validate your schema (perhaps as part of a pipeline)\n\n### Minimal Example\n\n```python\nfrom typing import Dict\nfrom aws_lambda_powertools.event_handler import APIGatewayRestResolve, Rresponse\nfrom aws_lambda_powertools.utilities.typing import LambdaContext\nfrom aws_lambda_powertools.middleware import validate_request\n\n\napp = APIGatewayRestResolver()\n\n@app.post("/example")\ndef example() -> Response:\n  ...\n\n@validate_request(oas_path="openapi.yaml")\ndef lambda_handler(event: Dict, context: LambdaContext) -> Dict:\n    response = app.resolve(event, context)\n\n    return response\n```\n\n\n### Extended Example\n```python\nfrom typing import Dict\nfrom aws_lambda_powertools.event_handler import APIGatewayRestResolve, Rresponse\nfrom aws_lambda_powertools.utilities.typing import LambdaContext\nfrom aws_lambda_powertools.middleware import validate_request\n\n\napp = APIGatewayRestResolver()\n\n@app.post("/example")\ndef example() -> Response:\n  ...\n\n@validate_request(\n  oas_path="openapi.yaml",\n  validate_spec=True,  # default True, disable to not validate OpenAPI Schema\n  validate_body=True,  # default True, disable to not validate event["body"] \n  validate_headers=True,  # default True, disable to not validate event["headers"]\n  validate_query=True,  # default True, disable to not validate event["rawQueryString"]\n  validate_path=True,  # default True, disable to not validate path parameters\n  validate_cookies=True, # default True, disable to not validate cookies\n\n)\ndef lambda_handler(event: Dict, context: LambdaContext) -> Dict:\n    response = app.resolve(event, context)\n\n    return response\n```\n\n',
    'author': 'Rasmus Hansen',
    'author_email': 'R.FangelHansen@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
